@@ -161,20 +161,17 @@ __lxcfs_fuse_ops int proc_open(const char *path, struct fuse_file_info *fi)
 	if (type == -1)
 		return -ENOENT;
 
-	info = malloc(sizeof(*info));
+	info = zalloc(sizeof(*info));
 	if (!info)
 		return -ENOMEM;
 
-	memset(info, 0, sizeof(*info));
 	info->type = type;
 
 	info->buflen = get_procfile_size(path) + BUF_RESERVE_SIZE;
 
-	info->buf = malloc(info->buflen);
+	info->buf = zalloc(info->buflen);
 	if (!info->buf)
 		return -ENOMEM;
-
-	memset(info->buf, 0, info->buflen);
 	/* set actual size to buffer size */
 	info->size = info->buflen;
 
@@ -210,7 +207,7 @@ static uint64_t get_memlimit(const char *cgroup, bool swap)
 		ret = cgroup_ops->get_memory_swap_max(cgroup_ops, cgroup, &memlimit_str);
 	else
 		ret = cgroup_ops->get_memory_max(cgroup_ops, cgroup, &memlimit_str);
-	if (ret > 0 && safe_uint64(memlimit_str, &memlimit, 10) < 0)
+	if (ret > 0 && memlimit_str[0] && safe_uint64(memlimit_str, &memlimit, 10) < 0)
 		lxcfs_error("Failed to convert memlimit %s", memlimit_str);
 
 	return memlimit;
@@ -247,12 +244,13 @@ static inline bool startswith(const char *line, const char *pref)
 static int proc_swaps_read(char *buf, size_t size, off_t offset,
 			   struct fuse_file_info *fi)
 {
-	__do_free char *cg = NULL, *memswlimit_str = NULL, *memusage_str = NULL,
-		       *memswusage_str = NULL;
+	__do_free char *cgroup = NULL, *memusage_str = NULL, *memswusage_str = NULL;
 	struct fuse_context *fc = fuse_get_context();
+	struct lxcfs_opts *opts = (struct lxcfs_opts *)fuse_get_context()->private_data;
+	bool wants_swap = opts && !opts->swap_off && liblxcfs_can_use_swap();
 	struct file_info *d = INTTYPE_TO_PTR(fi->fh);
 	uint64_t memswlimit = 0, memlimit = 0, memusage = 0, memswusage = 0,
-		 swap_total = 0, swap_free = 0;
+		 swtotal = 0, swfree = 0, swusage = 0;
 	ssize_t total_len = 0;
 	ssize_t l = 0;
 	char *cache = d->buf;
@@ -278,31 +276,37 @@ static int proc_swaps_read(char *buf, size_t size, off_t offset,
 	if (initpid <= 1 || is_shared_pidns(initpid))
 		initpid = fc->pid;
 
-	cg = get_pid_cgroup(initpid, "memory");
-	if (!cg)
+	cgroup = get_pid_cgroup(initpid, "memory");
+	if (!cgroup)
 		return read_file_fuse("/proc/swaps", buf, size, d);
-	prune_init_slice(cg);
+	prune_init_slice(cgroup);
 
-	memlimit = get_min_memlimit(cg, false);
+	memlimit = get_min_memlimit(cgroup, false);
 
-	ret = cgroup_ops->get_memory_current(cgroup_ops, cg, &memusage_str);
+	ret = cgroup_ops->get_memory_current(cgroup_ops, cgroup, &memusage_str);
 	if (ret < 0)
 		return 0;
 
 	if (safe_uint64(memusage_str, &memusage, 10) < 0)
 		lxcfs_error("Failed to convert memusage %s", memusage_str);
 
-	ret = cgroup_ops->get_memory_swap_max(cgroup_ops, cg, &memswlimit_str);
-	if (ret >= 0)
-		ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cg, &memswusage_str);
-	if (ret >= 0) {
-		memswlimit = get_min_memlimit(cg, true);
-
-		if (safe_uint64(memswusage_str, &memswusage, 10) < 0)
-			lxcfs_error("Failed to convert memswusage %s", memswusage_str);
-
-		swap_total = (memswlimit - memlimit) / 1024;
-		swap_free = (memswusage - memusage) / 1024;
+	if (wants_swap) {
+		memswlimit = get_min_memlimit(cgroup, true);
+		if (memswlimit > 0) {
+			ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cgroup, &memswusage_str);
+			if (ret >= 0 && safe_uint64(memswusage_str, &memswusage, 10) == 0) {
+				if (memlimit > memswlimit)
+					swtotal = 0;
+				else
+					swtotal = (memswlimit - memlimit) / 1024;
+				if (memusage > memswusage || swtotal == 0)
+					swusage = 0;
+				else
+					swusage = (memswusage - memusage) / 1024;
+				if (swtotal >= swusage)
+					swfree = swtotal - swusage;
+			}
+		}
 	}
 
 	total_len = snprintf(d->buf, d->size, "Filename\t\t\t\tType\t\tSize\tUsed\tPriority\n");
@@ -320,16 +324,16 @@ static int proc_swaps_read(char *buf, size_t size, off_t offset,
 
 		while (getline(&line, &linelen, f) != -1) {
 			if (startswith(line, "SwapTotal:"))
-				sscanf(line, "SwapTotal:      %8" PRIu64 " kB", &swap_total);
+				sscanf(line, "SwapTotal:      %8" PRIu64 " kB", &swtotal);
 			else if (startswith(line, "SwapFree:"))
-				sscanf(line, "SwapFree:      %8" PRIu64 " kB", &swap_free);
+				sscanf(line, "SwapFree:      %8" PRIu64 " kB", &swfree);
 		}
 	}
 
-	if (swap_total > 0) {
+	if (swtotal > 0) {
 		l = snprintf(d->buf + total_len, d->size - total_len,
 			     "none%*svirtual\t\t%" PRIu64 "\t%" PRIu64 "\t0\n",
-			     36, " ", swap_total, swap_free);
+			     36, " ", swtotal, swfree);
 		total_len += l;
 	}
 
@@ -665,10 +669,9 @@ static int proc_uptime_read(char *buf, size_t size, off_t offset,
 {
 	struct fuse_context *fc = fuse_get_context();
 	struct file_info *d = INTTYPE_TO_PTR(fi->fh);
-	double busytime = get_reaper_busy(fc->pid);
 	char *cache = d->buf;
-	ssize_t total_len = 0;
-	double idletime, reaperage;
+	ssize_t total_len = 0, ret = 0;
+	double busytime, idletime, reaperage;
 
 #if RELOADTEST
 	iwashere();
@@ -677,12 +680,12 @@ static int proc_uptime_read(char *buf, size_t size, off_t offset,
 	if (offset) {
 		int left;
 
-		if (!d->cached)
-			return 0;
-
 		if (offset > d->size)
 			return -EINVAL;
 
+		if (!d->cached)
+			return 0;
+
 		left = d->size - offset;
 		total_len = left > size ? size : left;
 		memcpy(buf, cache + offset, total_len);
@@ -696,19 +699,19 @@ static int proc_uptime_read(char *buf, size_t size, off_t offset,
 	 * get_reaper_busy() function.
 	 */
 	idletime = reaperage;
+	busytime = get_reaper_busy(fc->pid);
 	if (reaperage >= busytime)
 		idletime = reaperage - busytime;
 
-	total_len = snprintf(d->buf, d->buflen, "%.2lf %.2lf\n", reaperage, idletime);
-	if (total_len < 0 || total_len >= d->buflen)
+	ret = snprintf(d->buf, d->buflen, "%.2lf %.2lf\n", reaperage, idletime);
+	if (ret < 0 || ret >= d->buflen)
 		return read_file_fuse("/proc/uptime", buf, size, d);
+	total_len = ret;
 
-	d->size = (int)total_len;
 	d->cached = 1;
-
+	d->size = total_len;
 	if (total_len > size)
 		total_len = size;
-
 	memcpy(buf, d->buf, total_len);
 
 	return total_len;
@@ -780,6 +783,10 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 	if (!f)
 		return 0;
 
+	/* Skip first system cpu line. */
+	if (getline(&line, &linelen, f) < 0)
+		return log_error(0, "proc_stat_read read first line failed");
+
 	/*
 	 * Read cpuacct.usage_all for all CPUs.
 	 * If the cpuacct cgroup is present, it is used to calculate the container's
@@ -796,10 +803,6 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 		lxcfs_v("proc_stat_read failed to read from cpuacct, falling back to the host's /proc/stat");
 	}
 
-	//skip first line
-	if (getline(&line, &linelen, f) < 0)
-		return log_error(0, "proc_stat_read read first line failed");
-
 	while (getline(&line, &linelen, f) != -1) {
 		ssize_t l;
 		char cpu_char[10]; /* That's a lot of cores */
@@ -1012,14 +1015,15 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 			     struct fuse_file_info *fi)
 {
 	__do_free char *cgroup = NULL, *line = NULL, *memusage_str = NULL,
-		       *memswlimit_str = NULL, *memswusage_str = NULL;
+		       *memswusage_str = NULL;
 	__do_free void *fopen_cache = NULL;
 	__do_fclose FILE *f = NULL;
 	struct fuse_context *fc = fuse_get_context();
 	struct lxcfs_opts *opts = (struct lxcfs_opts *)fuse_get_context()->private_data;
+	bool wants_swap = opts && !opts->swap_off && liblxcfs_can_use_swap(), host_swap = false;
 	struct file_info *d = INTTYPE_TO_PTR(fi->fh);
 	uint64_t memlimit = 0, memusage = 0, memswlimit = 0, memswusage = 0,
-		 hosttotal = 0, swtotal = 0;
+		 hosttotal = 0, swfree = 0, swusage = 0, swtotal = 0;
 	struct memory_stat mstat = {};
 	size_t linelen = 0, total_len = 0;
 	char *cache = d->buf;
@@ -1052,41 +1056,46 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 
 	prune_init_slice(cgroup);
 
-	memlimit = get_min_memlimit(cgroup, false);
-
+	/* memory limits */
 	ret = cgroup_ops->get_memory_current(cgroup_ops, cgroup, &memusage_str);
 	if (ret < 0)
 		return read_file_fuse("/proc/meminfo", buf, size, d);
 
+	if (safe_uint64(memusage_str, &memusage, 10) < 0)
+		lxcfs_error("Failed to convert memusage %s", memusage_str);
+
 	if (!cgroup_parse_memory_stat(cgroup, &mstat))
 		return read_file_fuse("/proc/meminfo", buf, size, d);
 
+	memlimit = get_min_memlimit(cgroup, false);
+
 	/*
 	 * Following values are allowed to fail, because swapaccount might be
 	 * turned off for current kernel.
 	 */
-	ret = cgroup_ops->get_memory_swap_max(cgroup_ops, cgroup, &memswlimit_str);
-	if (ret >= 0)
-		ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cgroup, &memswusage_str);
-	if (ret >= 0) {
+	if (wants_swap) {
 		memswlimit = get_min_memlimit(cgroup, true);
-		memswlimit = memswlimit / 1024;
-
-		if (safe_uint64(memswusage_str, &memswusage, 10) < 0)
-			lxcfs_error("Failed to convert memswusage %s", memswusage_str);
-		memswusage = memswusage / 1024;
-		swtotal = memswlimit;
+		if (memswlimit > 0) {
+			ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cgroup, &memswusage_str);
+			if (ret >= 0 && safe_uint64(memswusage_str, &memswusage, 10) == 0) {
+				if (memlimit > memswlimit)
+					swtotal = 0;
+				else
+					swtotal = (memswlimit - memlimit) / 1024;
+				if (memusage > memswusage || swtotal == 0)
+					swusage = 0;
+				else
+					swusage = (memswusage - memusage) / 1024;
+			}
+		}
 	}
 
-	if (safe_uint64(memusage_str, &memusage, 10) < 0)
-		lxcfs_error("Failed to convert memusage %s", memusage_str);
-	memlimit /= 1024;
-	memusage /= 1024;
-
 	f = fopen_cached("/proc/meminfo", "re", &fopen_cache);
 	if (!f)
 		return read_file_fuse("/proc/meminfo", buf, size, d);
 
+	memusage /= 1024;
+	memlimit /= 1024;
 	while (getline(&line, &linelen, f) != -1) {
 		ssize_t l;
 		char *printme, lbuf[100];
@@ -1107,33 +1116,34 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 		} else if (startswith(line, "MemAvailable:")) {
 			snprintf(lbuf, 100, "MemAvailable:   %8" PRIu64 " kB\n", memlimit - memusage + mstat.total_cache / 1024);
 			printme = lbuf;
-		} else if (startswith(line, "SwapTotal:") && memswlimit > 0 && opts && opts->swap_off == false) {
-			uint64_t hostswtotal = 0;
+		} else if (startswith(line, "SwapTotal:")) {
+			if (wants_swap) {
+				uint64_t hostswtotal = 0;
 
-			sscanf(line + STRLITERALLEN("SwapTotal:"), "%" PRIu64, &hostswtotal);
+				sscanf(line + STRLITERALLEN("SwapTotal:"), "%" PRIu64, &hostswtotal);
 
-			/* Don't advertise more SWAP than the total memory allowed. */
-			if (hostswtotal < swtotal)
-				swtotal = hostswtotal;
+				if (hostswtotal < swtotal) {
+					swtotal = hostswtotal;
+					host_swap = true;
+				}
+			}
 
 			snprintf(lbuf, 100, "SwapTotal:      %8" PRIu64 " kB\n", swtotal);
 			printme = lbuf;
-		} else if (startswith(line, "SwapTotal:") && opts && opts->swap_off == true) {
-			snprintf(lbuf, 100, "SwapTotal:      %8" PRIu64 " kB\n", (uint64_t)0);
-			printme = lbuf;
-		} else if (startswith(line, "SwapFree:") && memswlimit > 0 &&
-			   opts && opts->swap_off == false) {
-			uint64_t swfree = 0;
-			uint64_t swusage = 0;
+		} else if (startswith(line, "SwapFree:")) {
+			if (wants_swap) {
+				uint64_t hostswfree = 0;
 
-			swusage = memswusage - memusage;
-			swfree = swtotal - swusage;
+				if (host_swap) {
+					sscanf(line + STRLITERALLEN("SwapFree:"), "%" PRIu64, &hostswfree);
+					swfree = hostswfree;
+				} else if (swtotal >= swusage) {
+					swfree = swtotal - swusage;
+				}
+			}
 
 			snprintf(lbuf, 100, "SwapFree:       %8" PRIu64 " kB\n", swfree);
 			printme = lbuf;
-		} else if (startswith(line, "SwapFree:") && opts && opts->swap_off == true) {
-			snprintf(lbuf, 100, "SwapFree:       %8" PRIu64 " kB\n", (uint64_t)0);
-			printme = lbuf;
 		} else if (startswith(line, "Slab:")) {
 			snprintf(lbuf, 100, "Slab:        %8" PRIu64 " kB\n", (uint64_t)0);
 			printme = lbuf;
