@@ -96,12 +96,14 @@ static int sys_devices_system_cpu_online_read(char *buf, size_t size,
 	if (use_view)
 		max_cpus = max_cpu_count(cg);
 
-	if (max_cpus == 0)
-		return read_file_fuse("/sys/devices/system/cpu/online", buf, size, d);
-	if (max_cpus > 1)
-		total_len = snprintf(d->buf, d->buflen, "0-%d\n", max_cpus - 1);
-	else
-		total_len = snprintf(d->buf, d->buflen, "0\n");
+	if (use_view) {
+		if (max_cpus > 1)
+			total_len = snprintf(d->buf, d->buflen, "0-%d\n", max_cpus - 1);
+		else
+			total_len = snprintf(d->buf, d->buflen, "0\n");
+	} else {
+		total_len = snprintf(d->buf, d->buflen, "%s\n", cpuset);
+	}
 	if (total_len < 0 || total_len >= d->buflen)
 		return log_error(0, "Failed to write to cache");
 
