@@ -124,10 +124,13 @@ static int in_same_namespace(pid_t pid1, pid_t pid2, const char *ns)
 
 bool is_shared_pidns(pid_t pid)
 {
+	__do_close int fd = -EBADF;
+
 	if (pid != 1)
 		return false;
 
-	if (in_same_namespace(pid, getpid(), "pid") == -EINVAL)
+	fd = in_same_namespace(pid, getpid(), "pid");
+	if (fd == EINVAL)
 		return true;
 
 	return false;
@@ -540,3 +543,42 @@ int safe_uint64(const char *numstr, uint64_t *converted, int base)
 	*converted = u;
 	return 0;
 }
+
+static int char_left_gc(const char *buffer, size_t len)
+{
+	size_t i;
+
+	for (i = 0; i < len; i++) {
+		if (buffer[i] == ' ' ||
+		    buffer[i] == '\t')
+			continue;
+
+		return i;
+	}
+
+	return 0;
+}
+
+static int char_right_gc(const char *buffer, size_t len)
+{
+	int i;
+
+	for (i = len - 1; i >= 0; i--) {
+		if (buffer[i] == ' '  ||
+		    buffer[i] == '\t' ||
+		    buffer[i] == '\n' ||
+		    buffer[i] == '\0')
+			continue;
+
+		return i + 1;
+	}
+
+	return 0;
+}
+
+char *trim_whitespace_in_place(char *buffer)
+{
+	buffer += char_left_gc(buffer, strlen(buffer));
+	buffer[char_right_gc(buffer, strlen(buffer))] = '\0';
+	return buffer;
+}
